/* This file was generated by syncqt. */
#ifndef QT_QTUIPLUGIN_VERSION_H
#define QT_QTUIPLUGIN_VERSION_H

#define QTUIPLUGIN_VERSION_STR "5.6.3"

#define QTUIPLUGIN_VERSION 0x050603

#endif // QT_QTUIPLUGIN_VERSION_H
