/* This file was generated by syncqt. */
#ifndef QT_QTWEBCHANNEL_VERSION_H
#define QT_QTWEBCHANNEL_VERSION_H

#define QTWEBCHANNEL_VERSION_STR "5.6.3"

#define QTWEBCHANNEL_VERSION 0x050603

#endif // QT_QTWEBCHANNEL_VERSION_H
